<?php
/**
Saves and retrieves contest entries from XML files.

This file could be adapted to manage contest entries in a MySQL database
instead, but for simplicity XML files are used.
 **/
class ContData
{
    public $key;
    public $person;
    public $email;
    public $age;
    public $loc;
    public $desc;
    public $imgw;
    public $imgh;
    public $tnw;
    public $tnh;
    public $date;
    public $ip;

    private $dir;
    /** @var DOMDocument */
    private $doc;

    public function __construct($dir)
    {
        $this->dir = $dir;
    }

    public function infofile()
    {
        return $this->dir . '/' . date('Ymd', $this->date) . '_' . $this->key . '.xml';
    }

    public function imgfile()
    {
        return $this->dir . '/' . date('Ymd', $this->date) . '_' . $this->key . '.jpeg';
    }

    public function tnfile()
    {
        return $this->dir . '/' . date('Ymd', $this->date) . '_' . $this->key . '.tn.jpeg';
    }

    /**
     * @param DOMElement $item
     * @param $fld
     * @param $val
     * @return void
     */
    private function writeField($item, $fld, $val)
    {
        $el = $this->doc->createElement($fld);
        $el->appendChild($this->doc->createTextNode($val));
        $item->appendChild($el);
    }

    /*
    Save the contest entry info. Must set date and key before using.
    File name is created from appropriate directory, date, and key.
    File name is returned.
    */
    public function save()
    {
        $this->doc = new DOMDocument();
        $this->doc->formatOutput = true;
        $item = $this->doc->createElement('item');
        $this->doc->appendChild($item);
        $this->writeField($item, FN_KEY, $this->key);
        $this->writeField($item, FN_DATE, $this->date);
        $this->writeField($item, FN_PERSON, $this->person);
        $this->writeField($item, FN_EMAIL, $this->email);
        $this->writeField($item, FN_AGE, $this->age);
        $this->writeField($item, FN_LOC, $this->loc);
        $this->writeField($item, FN_DESC, $this->desc);
        $this->writeField($item, FN_IP, $this->ip);
        $this->writeField($item, FN_IMGW, $this->imgw);
        $this->writeField($item, FN_IMGH, $this->imgh);
        $this->writeField($item, FN_TNW, $this->tnw);
        $this->writeField($item, FN_TNH, $this->tnh);
        $fn = $this->infofile();
        $this->doc->save($fn);
        return $fn;
    }

    public function post()
    {
        $this->person = req(FN_PERSON);
        $this->email = req(FN_EMAIL);
        $this->age = req(FN_AGE);
        $this->loc = req(FN_LOC);
        $this->desc = req(FN_DESC);
        $this->ip = req(FN_IP);
        $this->save();
    }

    /**
     * @param DOMElement $item
     * @param $fld
     * @return string
     */
    private function readField($item, $fld)
    {
        $el = $item->getElementsByTagName($fld)->item(0);
        return $el ? $el->nodeValue : '';
    }

    /*
    Load the contest entry info based on current date and key.
    Set date and key before using.
    Returns true on success, false on failure.
    */
    public function load()
    {
        return $this->loadFromFile($this->infofile());
    }

    /*
    Load contest entry info from file name.
    No need to set date and key as these are obtained from file.
    Returns true on success, false on failure.
    */
    public function loadFromFile($fn)
    {
        if (!file_exists($fn)) return false;
        $this->doc = new DOMDocument();
        $this->doc->load($fn);
        $item = $this->doc->getElementsByTagName('item')->item(0);
        $this->key = $this->readField($item, FN_KEY);
        $this->date = $this->readField($item, FN_DATE);
        $this->person = $this->readField($item, FN_PERSON);
        $this->email = $this->readField($item, FN_EMAIL);
        $this->age = $this->readField($item, FN_AGE);
        $this->loc = $this->readField($item, FN_LOC);
        $this->desc = $this->readField($item, FN_DESC);
        $this->ip = $this->readField($item, FN_IP);
        $this->imgw = $this->readField($item, FN_IMGW);
        $this->imgh = $this->readField($item, FN_IMGH);
        $this->tnw = $this->readField($item, FN_TNW);
        $this->tnh = $this->readField($item, FN_TNH);
        return true;
    }

    private function moveFile($src, $dst)
    {
        if (file_exists($src)) {
            return rename($src, $dst);
        } else {
            return false;
        }
    }

    public function moveImagesFrom($imgfile, $tnfile)
    {
        $r1 = $this->moveFile($imgfile, $this->imgfile());
        $r2 = $this->moveFile($tnfile, $this->tnfile());
        return $r1 && $r2;
    }

    /*
    Save changes and move to a different directory.
    */
    public function moveTo($dir)
    {
        // Get current file names
        $oldinfo = $this->infofile();
        $oldimg = $this->imgfile();
        $oldtn = $this->tnfile();

        // Change directory
        $this->dir = $dir;

        // Move files
        $r1 = $this->moveFile($oldinfo, $this->infofile());
        $r2 = $this->moveFile($oldimg, $this->imgfile());
        $r3 = $this->moveFile($oldtn, $this->tnfile());
        return $r1 && $r2 && $r3;
    }

    public function sendEntryEmail()
    {
        $msg = 'New contest entry for ' . CONTEST_TITLE . '...' . NL
               . selfPath() . '/' . 'contview.php?fn=' . $this->infofile() . '&secret=' . urlencode(CONTEST_SECRET);
        mail(CONTEST_EMAIL, CONTEST_SUBJECT, $msg);
    }

    public function toString()
    {
        return $this->doc->saveXML();
    }

    private static function getFiles($dir)
    {
        return glob($dir . '/*.xml');
    }

    private static function compare($a1, $a2)
    {
        foreach ($a1 as $v1) {
            list(, $v2) = each($a2);
            if ($v1 < $v2) return -1;
            if ($v1 > $v2) return 1;
        }
        return 0;
    }

    private static function sortAge($cont1, $cont2)
    {
        /* @var $cont1 ContData */
        /* @var $cont2 ContData */
        return ContData::compare(
            array($cont1->age, $cont1->person),
            array($cont2->age, $cont1->person)
        );
    }

    private static function sortDate($cont1, $cont2)
    {
        /* @var $cont1 ContData */
        /* @var $cont2 ContData */
        return ContData::compare(
            array($cont1->date, $cont1->person),
            array($cont2->date, $cont1->person)
        );
    }

    private static function getList($dir, $sort_function)
    {
        $files = ContData::getFiles($dir);
        $list = array();
        foreach ($files as $fn) {
            $cont = new ContData($dir);
            $cont->loadFromFile($fn);
            $list[] = $cont;
        }
        usort($list, array('ContData', $sort_function));
        return $list;
    }

    public static function showList($dir, $sort, $pg)
    {
        if ($sort == FN_AGE) {
            $sort_function = 'sortAge';
        } else {
            $sort_function = 'sortDate';
        }
        $list = ContData::getList($dir, $sort_function);
        $ntot = count($list);

        $ifrom = ($pg - 1) * PAGE_SIZE;
        $list = array_slice($list, $ifrom, PAGE_SIZE);

        if ($ntot == 0) {
            echo '<p>No pictures to display.</p>';
        } else {
            /* @var $cont ContData */
            foreach ($list as $cont) {
                $fn = $cont->infofile();
                $tn = $cont->tnfile();
                echo '<div class="tnlist">'
                     . '<a href="contview.php?fn=' . $fn . '&dir=' . $dir . '&secret=' . urlencode(CONTEST_SECRET) . '">'
                     . '<img src="' . $tn . '"></a><p>'
                     . $cont->nameAndAge() . '<br>'
                     . $cont->email . '<br>'
                     . $cont->ip . '<br>'
                     . '</p></div>';
            }
        }
        return $ntot;
    }

    private function nameAndAge()
    {
        $age = $this->age;
        $ret = $this->person;
        if ($age !== false) {
            $ret .= ', ' . $age . ' y.o.';
        }
        return $ret;
    }

    public function display()
    {
        echo '<img src="' . $this->imgfile() . '">';
        echo '<p class="disp"><b>by:</b> ' . $this->person . '</p>';
        $age = $this->age;
        if ($age !== false) {
            echo '<p class="disp"><b>age:</b> ' . $this->age . ' y.o.</p>';
        }
        if ($this->loc) {
            echo '<p class="disp"><b>from:</b> ' . $this->loc . '</p>';
        }
        if ($this->desc) {
            echo '<p class="disp"><b>comment:</b><br/>' . nl2br($this->desc) . '</p>';
        }
    }

    public static function exportDir($dir, $sort)
    {
        $list = ContData::getList($dir, $sort);
        /* @var $cont ContData */
        foreach ($list as $cont) {
            $desc = str_replace('"', '""', $cont->desc);
            $date = date('d/m/Y', $cont->date);
            $v = array($cont->person, $cont->email, $cont->age, $cont->loc, $desc, $cont->ip, $date, $cont->key, $dir);
            echo '"'.join('","', $v).'"'."\n";
        }
    }

    public static function export()
    {
        $queue = ContData::getList(DIR_QUEUE, "sortAge");
        $trash = ContData::getList(DIR_TRASH, "sortAge");

        header("Content-Type: application/csv");
        header("Content-Disposition: attachment; filename=contest.csv");

        echo FLD_PERSON . "," . FLD_EMAIL . "," . FLD_AGE . "," . FLD_LOC . "," . FLD_DESC . "," . FLD_IP . ",Date,Key,Dir\n";

        ContData::exportDir(DIR_ENTRY, "sortAge");
        ContData::exportDir(DIR_QUEUE, "sortAge");
        ContData::exportDir(DIR_TRASH, "sortAge");
    }

}

?>